@@ -20,6 +20,9 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
+
+#include <config.h>
+
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -29,9 +32,6 @@
 #include "libusb.h"
 #include "ezusb.h"
 
-extern void logerror(const char *format, ...)
-	__attribute__ ((format(printf, 1, 2)));
-
 /*
  * This file contains functions for uploading firmware into Cypress
  * EZ-USB microcontrollers. These chips use control endpoint 0 and vendor
